@@ -5,10 +5,12 @@ mod goban;
 mod goban_range;
 mod goban_style;
 mod make_svg;
+mod node_description;
 
-pub use errors::GobanSVGError;
+pub use errors::MakeSvgError;
 pub use generated_styles::GENERATED_STYLES;
-pub use goban::{Goban, NodeDescription, Stone, StoneColor};
+pub use goban::{Goban, Stone, StoneColor};
 pub use goban_range::GobanRange;
 pub use goban_style::GobanStyle;
 pub use make_svg::{make_svg, BoardSide, MakeSvgOptions};
+pub use node_description::{NodeDescription, NodeDescriptionError, NodePathStep};
